﻿Imports Microsoft.StylusInput
Imports Microsoft.Ink

Public Class SyncPlugin1
    Implements Microsoft.StylusInput.IStylusSyncPlugin

    Delegate Sub SetTextCallback(ByVal [text] As String)


    Public ReadOnly Property DataInterest() As Microsoft.StylusInput.DataInterestMask Implements Microsoft.StylusInput.IStylusSyncPlugin.DataInterest
        Get
            '種々のデータのうちどんな情報を使用可能にするか（IStylusSyncPluginで実装可能なメソッドのうち、どれを使用可能にするか）
            Return DataInterestMask.AllStylusData And Not DataInterestMask.SystemGesture
        End Get
    End Property

    Public Sub CustomStylusDataAdded(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.CustomStylusData) Implements Microsoft.StylusInput.IStylusSyncPlugin.CustomStylusDataAdded
        Debug.Print(Now & " CustomStylusDataAdded")
    End Sub

    Public Sub [Error](ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.ErrorData) Implements Microsoft.StylusInput.IStylusSyncPlugin.Error
        Debug.Print(Now & " Error")
    End Sub

    Public Sub InAirPackets(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.InAirPacketsData) Implements Microsoft.StylusInput.IStylusSyncPlugin.InAirPackets
        '入力可能領域でペン先スイッチoff時。ずっと、ペンを動かさなくても繰り返し繰り返し…
        'Debug.Print(Now & " InAirPackets")
    End Sub

    Public Sub Packets(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.PacketsData) Implements Microsoft.StylusInput.IStylusSyncPlugin.Packets
        '入力可能領域でペン先スイッチon時。ずっと、ペンを動かさなくても繰り返し繰り返し…
        'Debug.Print(Now & " Packets")

        ' 描画
        Debug.Assert(data.PacketPropertyCount >= 2)   ' 2～3種類。X, Y, Pressure
        Debug.Assert(data.Count Mod data.PacketPropertyCount = 0)
        For idx As Integer = 0 To data.Count - (1 * data.PacketPropertyCount) Step data.PacketPropertyCount
            Dim ink_pt As New Drawing.Point(data(idx), data(idx + 1))

            Dim t As Microsoft.Ink.Tablet = New Microsoft.Ink.Tablets().DefaultTablet

            Dim normalPressure As Microsoft.Ink.TabletPropertyMetrics = Nothing
            Dim pressure As Double                                              ' 筆圧(%)
            If data.PacketPropertyCount >= 3 Then
                normalPressure = t.GetPropertyMetrics(Microsoft.Ink.PacketProperty.NormalPressure)
                pressure = data(idx + 2) / normalPressure.Maximum
            Else
                pressure = 0.1                                                  ' 筆圧が無いときは筆圧固定
            End If

            Dim penWidth As Single = 50 * pressure                              ' 50px * 筆圧(%)
            If penWidth > 0 Then
                Dim pen As New Drawing.Pen(Drawing.Color.DarkBlue, penWidth)
                Using g As Graphics = Form1.getInstance().PictureBox1.CreateGraphics

                    ' インクスペース座標→ピクセル座標 変換
                    g.PageUnit = GraphicsUnit.Pixel
                    Dim r As New Renderer()
                    Dim p As Drawing.Point = New Drawing.Point(ink_pt.X, ink_pt.Y)
                    r.InkSpaceToPixel(g, p)

                    If data.PacketPropertyCount >= 3 Then
                        Me.SetText(String.Format("({0,7:D}inc, {1,7:D}inc) -> ({2,6:D}px, {3,6:D}px) ({4,7:P}, {5,7:P}), pressure:{7,6:D}/{6,6:D} ({8,7:P}), PropertyCount:{9,2:D}", ink_pt.X, ink_pt.Y, p.X, p.Y, p.X / ink_pt.X, p.Y / ink_pt.Y, normalPressure.Maximum, data(idx + 2), data(idx + 2) / normalPressure.Maximum, data.PacketPropertyCount.ToString()))
                    Else
                        Me.SetText(String.Format("({0,6:D}px, {1,6:D}px), PropertyCount:{2,2:D}", p.X, p.Y, data.PacketPropertyCount.ToString()))
                    End If
                    p.Offset(penWidth / -2.0F, penWidth / -2.0F)        'ポインター位置が中心になるよう描くため
                    g.DrawArc(pen, p.X, p.Y, penWidth, penWidth, 0, 360)
                End Using
            End If
        Next

    End Sub

    ' ラベル更新（外部スレッドからの更新）
    Private Sub SetText(ByVal [text] As String)
        Dim f As Form1 = Stylus1.Form1.getInstance()

        If f.InvokeRequired Then
            Dim d As New SetTextCallback(AddressOf SetText)
            f.Invoke(d, New Object() {[text]})
        Else
            f.Label1.Text = [text]
        End If
    End Sub

    Public Sub RealTimeStylusDisabled(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.RealTimeStylusDisabledData) Implements Microsoft.StylusInput.IStylusSyncPlugin.RealTimeStylusDisabled
        'アプリケーションの終了処理に
        Debug.Print(Now & " RealTimeStylusDisabled")
    End Sub

    Public Sub RealTimeStylusEnabled(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.RealTimeStylusEnabledData) Implements Microsoft.StylusInput.IStylusSyncPlugin.RealTimeStylusEnabled
        'アプリケーションの初期設定に
        Debug.Print(Now & " RealTimeStylusEnabled")
    End Sub

    Public Sub StylusButtonDown(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusButtonDownData) Implements Microsoft.StylusInput.IStylusSyncPlugin.StylusButtonDown
        'サイドスイッチ以外に、ペン先スイッチもボタンの1つになっている
        Debug.Print(Now & " StylusButtonDown")
    End Sub

    Public Sub StylusButtonUp(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusButtonUpData) Implements Microsoft.StylusInput.IStylusSyncPlugin.StylusButtonUp
        'サイドスイッチ以外に、ペン先スイッチもボタンの1つになっている
        Debug.Print(Now & " StylusButtonUp")
    End Sub

    Public Sub StylusDown(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusDownData) Implements Microsoft.StylusInput.IStylusSyncPlugin.StylusDown
        'ペン先スイッチon
        Debug.Print(Now & " StylusDown")
    End Sub

    Public Sub StylusUp(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusUpData) Implements Microsoft.StylusInput.IStylusSyncPlugin.StylusUp
        'ペン先スイッチoff
        Debug.Print(Now & " StylusUp")
    End Sub

    Public Sub StylusInRange(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusInRangeData) Implements Microsoft.StylusInput.IStylusSyncPlugin.StylusInRange
        'ペン先スイッチoffで入力可能領域にペンポインターが入ったとき
        Debug.Print(Now & " StylusInRange")
    End Sub

    Public Sub StylusOutOfRange(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.StylusOutOfRangeData) Implements Microsoft.StylusInput.IStylusSyncPlugin.StylusOutOfRange
        'ペン先スイッチoffで入力可能領域からペンポインターが出たとき
        Debug.Print(Now & " StylusOutOfRange")
    End Sub

    Public Sub SystemGesture(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.SystemGestureData) Implements Microsoft.StylusInput.IStylusSyncPlugin.SystemGesture
        'Debug.Print(Now & " SystemGesture")
    End Sub

    Public Sub TabletAdded(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.TabletAddedData) Implements Microsoft.StylusInput.IStylusSyncPlugin.TabletAdded
        Debug.Print(Now & " TabletAdded")
    End Sub

    Public Sub TabletRemoved(ByVal sender As Microsoft.StylusInput.RealTimeStylus, ByVal data As Microsoft.StylusInput.PluginData.TabletRemovedData) Implements Microsoft.StylusInput.IStylusSyncPlugin.TabletRemoved
        Debug.Print(Now & " TabletRemoved")
    End Sub

End Class
